Package: DiscreteTests
Type: Package
Title: Vectorised Computation of P-Values and Their Supports for
        Several Discrete Statistical Tests
Version: 0.5.0
Date: 2026-08-26
Authors@R: c(
    person("Florian", "Junge", role = c("cre", "aut"),
           email = "diso.fbmn@h-da.de",
           comment = c(ORCID = "0009-0001-6856-6938")),
    person("Christina", "Kihn", role = "aut"),
    person("Sebastian", "Döhler", role = "ctb",
           comment = c(ORCID = "0000-0002-0321-6355")),
    person("Guillermo", "Durand", role = "ctb",
           comment = c(ORCID = "0000-0003-4056-5631"))
  )
Description: Provides vectorised functions for computing p-values of various
  common discrete statistical tests, as described e.g. in Agresti (2002)
  <doi:10.1002/0471249688>, including their distributions. Exact and
  approximate computation methods are provided. For exact ones, several
  procedures of determining two-sided p-values are included, which are
  outlined in more detail in Hirji (2006) <doi:10.1201/9781420036190>.
License: GPL-3
Encoding: UTF-8
Language: en-GB
Imports: Rcpp, R6, checkmate, lifecycle, cli, tibble, withr
LinkingTo: Rcpp
URL: https://github.com/DISOhda/DiscreteTests
BugReports: https://github.com/DISOhda/DiscreteTests/issues
Config/roxygen2/version: 8.1.0
NeedsCompilation: yes
Packaged: 2026-08-26 14:07:16 UTC; fjunge
Author: Florian Junge [cre, aut] (ORCID:
    <https://orcid.org/0009-0001-6856-6938>),
  Christina Kihn [aut],
  Sebastian Döhler [ctb] (ORCID: <https://orcid.org/0000-0002-0321-6355>),
  Guillermo Durand [ctb] (ORCID: <https://orcid.org/0000-0003-4056-5631>)
Maintainer: Florian Junge <diso.fbmn@h-da.de>
Repository: CRAN
Date/Publication: 2026-08-26 15:10:02 UTC
