# Generated by roxygen2: do not edit by hand

S3method(summary,DiscreteTestResults)
export(DiscreteTestResults)
export(DiscreteTestResultsSummary)
export(binom.test.pv)
export(binom_test_pv)
export(fisher.test.pv)
export(fisher_test_pv)
export(homogeneity_test_pv)
export(mann_whitney_test_pv)
export(mcnemar.test.pv)
export(mcnemar_test_pv)
export(perm_test_pv)
export(poisson.test.pv)
export(poisson_test_pv)
export(sign_test_pv)
export(wilcox_test_pv)
import(Rcpp)
importFrom(R6,R6Class)
importFrom(checkmate,
  assert_character,
  assert_data_frame,
  assert_int,
  assert_integerish,
  assert_list,
  assert_logical,
  assert_numeric,
  assert_r6,
  assert_subset,
  qassert,
  qassertr
)
importFrom(cli,
  ansi_collapse,
  cli_abort,
  cli_end,
  cli_h1,
  cli_h2,
  cli_h3,
  cli_li,
  cli_text,
  cli_ul,
  cli_verbatim,
  cli_warn,
  col_blue,
  col_green,
  col_red,
  qty
)
importFrom(lifecycle,
  deprecate_stop,
  deprecated
)
importFrom(stats,
  dbinom,
  dhyper,
  dnorm,
  dpois,
  dsignrank,
  pchisq,
  pnorm,
  qnorm,
  qpois,
  setNames,
  stepfun
)
importFrom(tibble,
  add_column,
  as_tibble,
  rowid_to_column
)
importFrom(withr,with_options)
useDynLib(DiscreteTests)
