# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mann_whitney_probs_int <- function(m, n) {
    .Call('_DiscreteTests_mann_whitney_probs_int', PACKAGE = 'DiscreteTests', m, n)
}

mann_whitney_probs_ties_int <- function(ranks, size_x) {
    .Call('_DiscreteTests_mann_whitney_probs_ties_int', PACKAGE = 'DiscreteTests', ranks, size_x)
}

perm_test_run <- function(x, y, mu, method = "diff_mean", exact = TRUE, sims = 100000L) {
    .Call('_DiscreteTests_perm_test_run', PACKAGE = 'DiscreteTests', x, y, mu, method, exact, sims)
}

sign_rank_probs_int <- function(n) {
    .Call('_DiscreteTests_sign_rank_probs_int', PACKAGE = 'DiscreteTests', n)
}

sign_rank_probs_ties_int <- function(scores) {
    .Call('_DiscreteTests_sign_rank_probs_ties_int', PACKAGE = 'DiscreteTests', scores)
}

